import os
import unittest
from pathlib import Path
import io

from drb.drivers.file import DrbFileFactory

from drb.drivers.image import DrbImageFactory, DrbImageBaseNode
from drb.drivers.image.simple_node import DrbImageNodesValueNames

IMAGE = DrbImageNodesValueNames.IMAGE.value


class TestDrbNodeFactoryNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    image_fake = current_path / "files" / "fake.tiff"
    image_tif_one = current_path / "files" / 'GeogToWGS84GeoKey5.tif'
    image_png = current_path / "files" / 'png-248x300.png'
    image_jp2 = current_path / "files" / 'relax.jp2'
    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbImageFactory().create(self.node_file)
        return self.node

    def test_opened_file_node(self):
        node = self.open_node(str(self.image_tif_one))

        self.assertIsInstance(node, DrbImageBaseNode)
        self.assertEqual(node.name, self.node_file.name)
        self.assertEqual(node.namespace_uri, self.node_file.namespace_uri)

    def test_base_node(self):
        node = self.open_node(str(self.image_tif_one))

        self.assertEqual(node.parent, self.node_file.parent)
        self.assertEqual(node.value, self.node_file.value)

        self.assertIsInstance(node, DrbImageBaseNode)

        self.assertEqual(len(node), 8)
        self.assertTrue(node.has_child())
        self.assertEqual(len(node.children), 8)

    def test_base_node_attribute(self):
        node = self.open_node(str(self.image_tif_one))

        self.assertEqual(node.attributes, self.node_file.attributes)
        self.assertEqual(node.get_attribute('mode'),
                         self.node_file.get_attribute(
                             'mode'))

    def test_base_node_impl(self):
        node = self.open_node(str(self.image_tif_one))

        impl_base_file = io.BufferedIOBase

        self.assertTrue(node.has_impl(impl_base_file))

        impl = node.get_impl(impl_base_file)
        self.assertIsNotNone(impl)
        self.assertIsInstance(impl, impl_base_file)
        impl.close()

    def test_first_group(self):
        node = self.open_node(str(self.image_tif_one))

        self.assertIsInstance(node, DrbImageBaseNode)
        root_node = node[0]
        self.assertIsNotNone(root_node)
        root_node.close()



import copy
import io
from typing import Any, List, Dict, Tuple

import numpy
import rasterio as rasterio
import xarray
from deprecated.classic import deprecated
from rasterio.io import MemoryFile

from drb.core import DrbNode
from drb.nodes.abstract_node import AbstractNode
from drb.exceptions.core import DrbNotImplementationException
from drb.core.factory import DrbFactory
from drb.core.path import ParsedPath
from drb.topics.resolver import resolve_children

from .simple_node import DrbImageSimpleValueNode, DrbImageNodesValueNames
from .list_node import DrbImageListNode
from drb.exceptions.image import DrbImageNodeException


class DrbImageBaseNode(AbstractNode):

    def __init__(self, base_node: DrbNode):
        super().__init__()

        self.base_node = base_node
        self._data_set = None
        self._data_set_file_source = None
        self._data_xarray = None
        self.parent = self.base_node.parent
        self.name = self.base_node.name
        self.namespace_uri = self.base_node.namespace_uri
        self.value = self.base_node.value
        self._children: List[DrbNode] = None
        self._impl_mng = copy.copy(base_node._impl_mng)
        self.add_impl(rasterio.DatasetReader, self._get_rasterio_impl)
        self.add_impl(numpy.ndarray, self._get_numpy_ndarray_impl)
        self.add_impl(xarray.DataArray, self._get_xarray_impl)

    @staticmethod
    def _get_rasterio_impl(node: DrbNode, **kwargs):
        if isinstance(node, DrbImageBaseNode):
            return node._get_data_set()
        raise TypeError(f'Invalid node type: {type(node)}')

    @staticmethod
    def _get_numpy_ndarray_impl(node: DrbNode, **kwargs):
        if isinstance(node, DrbImageBaseNode):
            return node._get_data_set().read()
        raise TypeError(f'Invalid node type: {type(node)}')

    @staticmethod
    def _get_xarray_impl(node: DrbNode, **kwargs):
        if isinstance(node, DrbImageBaseNode):
            return node._get_data_xarray()
        raise TypeError(f'Invalid node type: {type(node)}')

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError

    @property
    def path(self) -> ParsedPath:
        return self.base_node.path

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the @ operator is recommended')
    def attributes(self) -> Dict[Tuple[str, str], Any]:
        return self.base_node.attributes

    @deprecated(version='1.2.0',
                reason='Usage of the @ operator is recommended')
    def get_attribute(self, name: str, namespace_uri: str = None) -> Any:
        return self.base_node.get_attribute(name, namespace_uri)

    def close(self):
        if self._data_set is not None:
            self._data_set.close()
        if self._data_set_file_source is not None:
            self._data_set_file_source.close()
        if self._data_xarray is not None:
            self._data_xarray.close()
        self.base_node.close()

    def _get_data_set(self) -> rasterio.DatasetReader:
        """
        Retrieve the data set of the image.

        Returns:
            rasterio.DatasetReader:  the data set of the image
        """
        if self._data_set is None:
            self._get_file_impl()
            self._memory_file = MemoryFile(
                file_or_bytes=self._data_set_file_source)
            self._data_set = self._memory_file.open()

        return self._data_set

    def _get_data_xarray(self) -> rasterio.DatasetReader:
        """
        Retrieve the data as Xarray of the image.

        Returns:
            rasterio.DatasetReader:  the data as Xarray of the image
        """
        if self._data_xarray is None:
            self._get_data_set()
            self._data_xarray = xarray.open_rasterio(
                self._data_set)
        return self._data_xarray

    def _get_file_impl(self):
        """
        retrieve the implementation of the file corresponding to his parent.
        """
        if self._data_set_file_source is None:
            if self.base_node.has_impl(io.BufferedIOBase):
                self._data_set_file_source = self.base_node \
                    .get_impl(io.BufferedIOBase)
            elif self.base_node.has_impl(io.BytesIO):
                self._data_set_file_source = self.base_node \
                    .get_impl(io.BytesIO)
            else:
                raise DrbImageNodeException(f'Unsupported parent '
                                            f'{type(self.parent).__name__} '
                                            f'for DrbImageNode')

    def _add_node_value(self, node_name, value):
        """
        Add a DrbImageSimpleValueNode node the list of children

        Parameters:
            node_name (str): The node name.
            value (any): the value corresponding to the name.
        """
        node_value = DrbImageSimpleValueNode(self, node_name, value)
        self._children.append(node_value)

    def _add_node_value_from_dict(self, node_name, dictionary, key):
        """
        Add a DrbImageSimpleValueNode node the list of children

        Parameters:
            node_name (str): The node name.
            dictionary (Dict): The dict containing the value.
            key (str): the key of the value.
        """
        if key in dictionary:
            value = dictionary[key]
        else:
            value = None
        self._add_node_value(node_name, value)

    def _add_values_from_dict(self, list_name, dictionary):
        """
        Add a DrbImageListNode node to the list of children

        Parameters:
            list_name (str): The name of the node.
            dictionary (Dict): The dict containing the value.
        """
        list_node = DrbImageListNode(self, list_name)
        for node_name, value in dictionary.items():
            node_value = DrbImageSimpleValueNode(list_node, node_name, value)
            list_node.append_child(node_value)

        self._children.append(list_node)

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    @resolve_children
    def children(self) -> List[DrbNode]:
        """
        Initiate the list of children containing some metadata,
        and the data of the image.

        Returns:
            List[DrbNode]: The list of children
        """
        if self._children is None:
            self._children = []
            data_set = self._get_data_set()

            self._add_node_value_from_dict(DrbImageNodesValueNames.FORMAT
                                           .value, data_set.meta, 'driver')

            self._add_node_value(DrbImageNodesValueNames.WIDTH.value,
                                 data_set.width)
            self._add_node_value(DrbImageNodesValueNames.HEIGHT.value,
                                 data_set.height)
            self._add_node_value(DrbImageNodesValueNames.NUM_BANDS.value,
                                 data_set.count)
            self._add_node_value_from_dict(DrbImageNodesValueNames.TYPE.value,
                                           data_set.meta, 'dtype')
            self._add_node_value_from_dict(DrbImageNodesValueNames.CRS.value,
                                           data_set.meta, 'crs')

            if data_set.tags() is not None:
                self._add_values_from_dict(DrbImageNodesValueNames.TAGS.value,
                                           data_set.tags())
            if data_set.tags() is not None:
                self._add_values_from_dict(DrbImageNodesValueNames.META.value,
                                           data_set.meta)
        return self._children


class DrbImageFactory(DrbFactory):

    def _create(self, node: DrbNode) -> DrbNode:
        if isinstance(node, DrbImageBaseNode):
            return node
        return DrbImageBaseNode(base_node=node)



import io
import os
import unittest
from pathlib import Path

import numpy
import rasterio
import xarray
from drb.exceptions.core import DrbNotImplementationException, DrbException
from drb.drivers.file import DrbFileFactory

from drb.drivers.image import DrbImageFactory, DrbImageBaseNode
from drb.drivers.image.simple_node import DrbImageNodesValueNames


class TestDrbTifNode(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    image_fake = current_path / "files" / "fake.tiff"
    image_tif_one = current_path / "files" / 'GeogToWGS84GeoKey5.tif'

    image_png = current_path / "files" / 'png-248x300.png'
    image_jp2_no_geo = current_path / "files" / 'relax.jp2'
    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbImageFactory().create(self.node_file)
        return self.node

    def test_format_name(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertIsInstance(root_node, DrbImageBaseNode)
        self.assertIsNone(root_node.namespace_uri)

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]
        self.assertIsNotNone(node_value)
        self.assertEqual(node_value.value, 'GTiff')
        root_node.close()

    def test_shape(self):
        root_node = self.open_node(str(self.image_png))

        node_value = root_node[DrbImageNodesValueNames.WIDTH.value]
        self.assertIsNotNone(node_value)
        self.assertEqual(node_value.value, 248)

        node_value = root_node[DrbImageNodesValueNames.HEIGHT.value]

        self.assertIsNotNone(node_value)
        self.assertEqual(node_value.value, 300)
        root_node.close()

    def test_type_band_count(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.TYPE.value]
        self.assertIsNotNone(node_value)
        self.assertEqual(node_value.value, 'uint8')

        node_value = root_node[DrbImageNodesValueNames.NUM_BANDS.value]
        self.assertIsNotNone(node_value)
        self.assertEqual(node_value.value, 1)
        root_node.close()

    def test_image_attributes(self):
        root_node = self.open_node(str(self.image_tif_one))

        list_attributes = root_node.attributes

        self.assertEqual(len(list_attributes), len(self.node_file.attributes))

        with self.assertRaises(KeyError):
            list_attributes[('phase_fitting_1', 'test')]
        with self.assertRaises(DrbException):
            root_node.get_attribute('test', None)
        root_node.close()

    def test_image_parent(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertEqual(root_node.parent, self.node_file.parent)
        level1 = root_node[0]

        self.assertEqual(level1.parent, root_node)
        root_node.close()

    def test_image_value(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertIsNone(root_node.value)
        root_node.close()

    def test_image_impl(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertTrue(root_node.has_impl(rasterio.DatasetReader))

        impl = root_node.get_impl(rasterio.DatasetReader)

        self.assertIsInstance(impl, rasterio.DatasetReader)
        read = impl.read()

        self.assertIsInstance(read, numpy.ndarray)

        impl_numpy = root_node.get_impl(numpy.ndarray)
        self.assertIsInstance(impl_numpy, numpy.ndarray)

        root_node.close()
        impl.close()

    def test_image_impl_not_supported(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertFalse(root_node.has_impl(io.BufferedRandom))

        with self.assertRaises(DrbNotImplementationException):
            root_node.get_impl(io.BufferedRandom)
        root_node.close()

    def test_image_with_crs(self):
        root_node = self.open_node(str(self.image_tif_one))

        crs_node = root_node[DrbImageNodesValueNames.CRS.value]
        self.assertIsNotNone(crs_node)

        self.assertIsNotNone(crs_node.value)

        self.assertIsNotNone(rasterio.crs.CRS.from_dict(crs_node
                                                        .value.to_dict()))

        crs_node.close()
        root_node.close()

    def test_image_without_crs(self):
        root_node = self.open_node(str(self.image_png))

        crs_node = root_node[DrbImageNodesValueNames.CRS.value]
        self.assertIsNotNone(crs_node)

        self.assertIsNone(crs_node.value)

        crs_node.close()
        root_node.close()

    def test_image_3_bands(self):
        root_node = self.open_node(str(self.image_jp2_no_geo))

        num_bands_node = root_node[DrbImageNodesValueNames.NUM_BANDS.value]
        self.assertIsNotNone(num_bands_node)

        self.assertEqual(num_bands_node.value, 3)

        root_node.close()

    def test_image_unknown_tags(self):
        root_node = self.open_node(str(self.image_png))

        fake_dict = {
            'width': 140,
            'height': 180
        }
        self.assertTrue(root_node.has_child())
        root_node._add_node_value_from_dict('Format1', fake_dict, 'driver')
        format1_node = root_node['Format1']
        self.assertIsNone(format1_node.value)

        root_node.close()

    def test_image_impl_xarray(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertTrue(root_node.has_impl(rasterio.DatasetReader))

        impl = root_node.get_impl(xarray.DataArray)

        self.assertIsInstance(impl, xarray.DataArray)

        self.assertEqual(len(impl.coords['band']), 1)
        self.assertEqual(len(impl.x), 101)
        self.assertEqual(len(impl.y), 101)

        root_node.close()
        impl.close()

    def test_image_impl_xarray_and_rasterio_togehter(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertTrue(root_node.has_impl(rasterio.DatasetReader))

        impl = root_node.get_impl(xarray.DataArray)

        self.assertIsInstance(impl, xarray.DataArray)

        array_data = impl.data
        self.assertIsInstance(array_data, numpy.ndarray)

        impl_numpy = root_node.get_impl(numpy.ndarray)
        self.assertIsInstance(impl_numpy, numpy.ndarray)

        self.assertListEqual(list(array_data[0][10]), list(impl_numpy[0][10]))

        root_node.close()
        impl.close()



import os
import unittest
from pathlib import Path

import numpy
import rasterio
from drb.drivers.file import DrbFileFactory

from drb.drivers.image import DrbImageFactory
from drb.drivers.image.simple_node import DrbImageNodesValueNames

IMAGE = DrbImageNodesValueNames.IMAGE.value


class TestDrbTifNode(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    # image_cog = current_path / "files" / "cog.tif"
    image_jpeg = current_path / "files" / "sample_640×426.jpeg"
    image_gif = current_path / "files" / "file_example_GIF_500kB.gif"
    image_webp = current_path / "files" / "file_example_WEBP_50kB.webp"
    image_bmp = current_path / "files" / "w3c_home_256.bmp"
    image_blx = current_path / "files" / "s4103.blx"
    image_bsb = current_path / "files" / "australia4c.kap"
    image_dted = current_path / "files" / "n43_wgs72.dt0"
    image_elas = current_path / "files" / "byte_elas.bin"
    image_gpkg = current_path / "files" / "byte.gpkg"
    image_kakadu = current_path / "files" / "rgbwcmyk01_YeGeo_kakadu.jp2"
    image_jdem = current_path / "files" / "fakejdem.mem"

    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbImageFactory().create(self.node_file)
        return self.node

    def open_image_check_dim_and_impl(self, path_file, width_waited,
                                      height_waited):
        root_node = self.open_node(str(path_file))

        self.assertTrue(root_node.has_impl(rasterio.DatasetReader))

        impl = root_node.get_impl(rasterio.DatasetReader)

        self.assertEqual(root_node['width'].value, width_waited)
        self.assertEqual(root_node['height'].value, height_waited)

        self.assertIsInstance(impl, rasterio.DatasetReader)
        read = impl.read()

        self.assertIsInstance(read, numpy.ndarray)

        impl_numpy = root_node.get_impl(numpy.ndarray)
        self.assertIsInstance(impl_numpy, numpy.ndarray)

        root_node.close()
        impl.close()

    # def test_image_cog_impl(self):
    #     self.open_image_check_dim_and_impl(self.image_cog, 16563, 16191)

    def test_image_jpeg_impl(self):
        self.open_image_check_dim_and_impl(self.image_jpeg, 640, 426)

    def test_image_gif_impl(self):
        self.open_image_check_dim_and_impl(self.image_gif, 1900, 1267)

    def test_image_webp_impl(self):
        self.open_image_check_dim_and_impl(self.image_webp, 1050, 700)

    def test_image_bmp_impl(self):
        self.open_image_check_dim_and_impl(self.image_bmp, 72, 48)

    def test_image_blx_impl(self):
        self.open_image_check_dim_and_impl(self.image_blx, 512, 512)

    def test_image_bsb_impl(self):
        self.open_image_check_dim_and_impl(self.image_bsb, 625, 480)

    def test_image_dted_impl(self):
        self.open_image_check_dim_and_impl(self.image_dted, 121, 121)

    def test_image_elas_impl(self):
        self.open_image_check_dim_and_impl(self.image_elas, 20, 20)

    def test_image_GPKG_impl(self):
        self.open_image_check_dim_and_impl(self.image_gpkg, 20, 20)

    def test_image_kakadu_impl(self):
        self.open_image_check_dim_and_impl(self.image_kakadu, 800, 100)

    def test_image_jdem_impl(self):
        self.open_image_check_dim_and_impl(self.image_jdem, 2, 2)



import os
import io
import posixpath
import unittest
from pathlib import Path

import numpy
import drb
from drb.exceptions.core import DrbNotImplementationException, DrbException
from drb.drivers.file import DrbFileFactory

from drb.drivers.image import DrbImageFactory
from drb.drivers.image.simple_node import DrbImageNodesValueNames


class TestDrbListNodeImage(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    image_fake = current_path / "files" / "fake.tiff"
    image_tif_one = current_path / "files" / 'GeogToWGS84GeoKey5.tif'
    image_png = current_path / "files" / 'png-248x300.png'
    image_jp2 = current_path / "files" / 'relax.jp2'

    def test_list_children(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))

        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.META.value]

        self.assertTrue(tags_list.has_child())
        self.assertIsNotNone(tags_list['transform'])

        self.assertIsNotNone(tags_list[-1])
        root_node.close()

    def test_list_parent(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))
        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.TAGS.value]
        self.assertEqual(tags_list.parent, root_node)
        self.assertEqual(tags_list.name, DrbImageNodesValueNames.TAGS.value)

        root_node.close()

    def test_list_namespace_uri(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))

        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.TAGS.value][0]
        self.assertIsNone(tags_list.namespace_uri)
        root_node.close()

    def test_list_value(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))

        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.TAGS.value]

        self.assertIsNone(tags_list.value)
        root_node.close()

    def test_list_impl_not_supported(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))

        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.TAGS.value]

        self.assertFalse(tags_list.has_impl(io.BufferedIOBase))
        self.assertFalse(tags_list.has_impl(numpy.ndarray))

        with self.assertRaises(DrbNotImplementationException):
            tags_list.get_impl(io.BufferedIOBase)
        root_node.close()

    def test_list_no_attributes(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))

        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.TAGS.value]
        self.assertFalse(bool(tags_list.attributes))
        with self.assertRaises(DrbException):
            tags_list.get_attribute('toto', None)
        root_node.close()

    def test_path(self):
        node_file = DrbFileFactory().create(str(self.image_tif_one))

        root_node = DrbImageFactory().create(node_file)

        tags_list = root_node[DrbImageNodesValueNames.META.value]

        self.assertEqual(
            tags_list.path.path,
            Path(self.image_tif_one).
            joinpath(DrbImageNodesValueNames.META.value).as_posix()
        )

        root_node.close()
        node_file.close()



from typing import List

from deprecated.classic import deprecated
from drb.core.node import DrbNode
from drb.nodes.abstract_node import AbstractNode
from drb.topics.resolver import resolve_children


class DrbImageListNode(AbstractNode):
    """
    This node is used to have one or many children of DrbNode but no value.
    Usually it will be a list of DrbImageSimpleValueNode.

    Parameters:
        parent (DrbNode): The node parent.
        name (str): the name of the data (usually a
                    value of DrbImageNodesValueNames)
    """
    def __init__(self, parent: DrbNode, name: str):
        super().__init__()

        self.name = name
        self.parent: DrbNode = parent
        self._children: List[DrbNode] = []

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    @resolve_children
    def children(self) -> List[DrbNode]:
        """
        Return a list of DrbNode representing the children of this node.

        Returns:
            List[DrbNode]: The children of this node.
        """
        return self._children

    def append_child(self, node: DrbNode) -> None:
        """
        Appends a DrbNode giving in argument to the list of children.

        Parameters:
            node (DrbNode): The node to add.
        """
        self._children.append(node)



import enum
from typing import List

from deprecated.classic import deprecated
from drb.core.node import DrbNode
from drb.nodes.abstract_node import AbstractNode


class DrbImageNodesValueNames(enum.Enum):
    """
    This enum  represent all metadata as heigth width ...
    """
    IMAGE = 'image'
    TAGS = 'tags'
    FORMAT = 'FormatName'
    WIDTH = 'width'
    HEIGHT = 'height'
    NUM_BANDS = 'NumBands'
    TYPE = 'Type'
    BOUNDARIES = 'Boundaries'
    CRS = 'crs'
    META = 'meta'


class DrbImageSimpleValueNode(AbstractNode):
    """
    This node is used to get simple value such as metadata
    an access the image data,
    usually the first child of the node.

    Parameters:
        parent (DrbNode): The node.
        name (str): the name of the data (usually
                    a value of DrbImageNodesValueNames)
        value (any): the value corresponding to the name.
    """

    def __init__(self, parent: DrbNode, name: str, value: any):
        super().__init__()
        self.name = name
        self.value = value
        self.parent: DrbNode = parent

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    def children(self) -> List[DrbNode]:
        """
        Not use in this class.

        Returns:
            List: an empty dict ([])
        """
        return []



import io
import os
import posixpath
import unittest
from pathlib import Path

import rasterio
from drb.exceptions.core import DrbException
from drb.drivers.file import DrbFileFactory

from drb.drivers.image import DrbImageFactory
from drb.drivers.image.simple_node import DrbImageNodesValueNames

GROUP_NOT_EXIST = "fake_group"


class TestDrbValueNodeImage(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    image_fake = current_path / "files" / "fake.tiff"
    image_tif_one = current_path / "files" / 'GeogToWGS84GeoKey5.tif'
    image_png = current_path / "files" / 'png-248x300.png'
    image_jp2 = current_path / "files" / 'relax.jp2'
    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbImageFactory().create(self.node_file)
        return self.node

    def test_value_attributes(self):
        root_node = self.open_node(str(self.image_tif_one))

        self.assertIsNone(root_node.namespace_uri)

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]
        self.assertIsNotNone(node_value)
        self.assertEqual(node_value.value, 'GTiff')
        attributes = node_value.attributes
        self.assertEqual(len(attributes), 0)
        with self.assertRaises(DrbException):
            self.assertIsNotNone(node_value.get_attribute('toto', None))

    def test_value_parent(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]
        self.assertEqual(node_value.parent, root_node)

    def test_value_impl(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]

        self.assertFalse(node_value.has_impl(rasterio.DatasetReader))
        self.assertFalse(node_value.has_impl(io.BufferedIOBase))

        with self.assertRaises(DrbException):
            node_value.get_impl(io.BufferedIOBase)

    def test_variable_namespace_uri(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]
        self.assertIsNone(node_value.namespace_uri)

    def test_variable_get_child_at(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]

        with self.assertRaises(IndexError):
            node_value[-2]
            node_value[0]
        with self.assertRaises(KeyError):
            node_value['Name']

    def test_variable_get_first_last_children(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]

        self.assertEqual(len(node_value), 0)
        self.assertEqual(node_value.children, [])
        self.assertFalse(node_value.has_child())

    def test_path(self):
        root_node = self.open_node(str(self.image_tif_one))

        node_value = root_node[DrbImageNodesValueNames.FORMAT.value]

        self.assertEqual(
            node_value.path.path,
            Path(self.image_tif_one).
            joinpath(DrbImageNodesValueNames.FORMAT.value).as_posix()
            )



from .base_node import DrbImageBaseNode, DrbImageSimpleValueNode, \
    DrbImageFactory, DrbImageListNode

from . import _version
__version__ = _version.get_versions()['version']

__all__ = [
    'DrbImageFactory',
    'DrbImageBaseNode',
    'DrbImageListNode',
    'DrbImageSimpleValueNode',
]



import os
import sys
import unittest
import uuid
from pathlib import Path

from drb.core.factory import FactoryLoader
from drb.nodes.logical_node import DrbLogicalNode
from drb.topics.dao import ManagerDao
from drb.topics.topic import TopicCategory

from drb.drivers.image import DrbImageFactory


class TestDrbImageSignature(unittest.TestCase):
    fc_loader = None
    topic_loader = None
    file_id = uuid.UUID('b7e03ac0-2b62-11ec-8d3d-0242ac130003')
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    image_tif_one = current_path / "files" / 'GeogToWGS84GeoKey5.tif'
    image_png = current_path / "files" / 'png-248x300.png'
    image_jp2 = current_path / "files" / 'relax.jp2'
    empty_file = current_path / "files" / 'empty_file'

    @classmethod
    def setUpClass(cls) -> None:
        cls.fc_loader = FactoryLoader()
        cls.topic_loader = ManagerDao()

    def test_impl_loading(self):
        factory_name = 'image'

        factory = self.fc_loader.get_factory(factory_name)
        self.assertIsNotNone(factory)
        self.assertIsInstance(factory, DrbImageFactory)

        topic = self.topic_loader.get_drb_topic(self.file_id)
        self.assertIsNotNone(factory)
        self.assertEqual(self.file_id, topic.id)
        self.assertEqual('image', topic.label)
        self.assertIsNone(topic.description)
        self.assertEqual(TopicCategory.FORMATTING, topic.category)
        self.assertEqual(factory_name, topic.factory)

    def test_impl_signatures(self):
        topic = self.topic_loader.get_drb_topic(self.file_id)

        node = DrbLogicalNode(self.image_tif_one)
        self.assertTrue(topic.matches(node))

        node = DrbLogicalNode(self.image_png)
        self.assertTrue(topic.matches(node))

        node = DrbLogicalNode(self.image_jp2)
        self.assertTrue(topic.matches(node))

        node = DrbLogicalNode('https://gitlab.com/drb-python')
        self.assertFalse(topic.matches(node))
